--- bogofilter-0.9.1.2/lexer.l.patch-lexer	2002-11-26 01:23:14 +0300
+++ bogofilter-0.9.1.2/lexer.l	2002-12-08 18:53:07 +0300
@@ -20,9 +20,12 @@
  * Our lexical analysis is different from Paul Graham's rules: 
  *
  * We throw away headers that are readily identifiable as dates.
- * We throw away text lines that look like BASE64 encoding.
+ * We throw away headers that are known to carry unique IDs
+ * We throw away text lines that look like BASE64 encoding or PGP
+ * signature fragments.
  * We throw away all digit strings that don't look like IP address parts.
  * We thow away lines beginning with <tab>id<space> -- mailer UDs.
+ * Some other id-lookalike occurrencies are also thrown away.
  * We throw away various bits of MIME cruft (things that look like
  * declarations and boundaries for multipart declarations).
  * We throw away *all* tokens of length 1 or 2.
@@ -31,6 +34,9 @@
  * The big win is recognizing machine-generated unique IDs that
  * we'll never see again and shouldn't 
  *
+ * Header names are tokenized together with trailing colon
+ * to separate them from ordinary words in the lists.
+ *
  * We don't treat dot between two alphanumerics as a separator,
  * because we want to keep domain names and IP addresses together as 
  * recognizable units. 
@@ -58,191 +64,356 @@
 
 char *yylval;
 static char alt_text[MAXTOKENLEN];
-static int past_header;
 static int yyinput(char *buf, int max_size);
 
 %}
 
 %option align nounput noyywrap noreject 8bit
 
-BASE64		^([A-Za-z0-9/+]+={1,2}|[A-Za-z0-9/+]{32,})$
+%s HEADER UNWANTED
+
+B64CHAR		[A-Za-z0-9/+]
+BASE64		^({B64CHAR}{4})*({B64CHAR}{4}|{B64CHAR}{3}=|{B64CHAR}{2}==)$
+PGPSIG		^={B64CHAR}{4}$
 UINT8		([01]?[0-9]?[0-9]|2([0-4][0-9]|5[0-5]))
 IPADDR		{UINT8}\.{UINT8}\.{UINT8}\.{UINT8}
-MIME_BOUNDARY	^--[^[:blank:][:cntrl:]]+
+MIME_BOUNDARY	^--[ A-Za-z0-9'()+_,./:=?-]*[A-Za-z0-9'()+_,./:=?-]
 
 %%
 
-abbr	;
-above	;
-acronym	;
-accesskey	;
-align	;
-all	;
-alt	;
-alink	;
-applet	;
-archive	;
-axis	;
-basefont	;
-baseline	;
-below	;
-bgcolor	;
-big	;
-body	;
-border	;
-bottom	;
-box	;
-button	;
-cellpadding	;
-cellspacing	;
-center	;
-char	;
-charoff	;
-charset	;
-circle	;
-cite	;
-colspan	;
-coords	;
-class	;
-classid	;
-clear	;
-codebase	;
-codetype	;
-color	;
-cols	;
-compact	;
-content	;
-datetime	;
-declare	;
-defer	;
-data	;
-default	;
-dfn	;
-dir	;
-disabled	;
-face	;
-font	;
-frameborder	;
-groups	;
-head	;
-headers	;
-height	;
-href	;
-hreflang	;
-hsides	;
-http-equiv	;
-hspace	;
-iframe	;
-input	;
-img	;
-ismap	;
-justify	;
-kbd	;
-label	;
-lang	;
-language	;
-left	;
-lhs	;
-link	;
-longdesc	;
-map	;
-marginheight	;
-marginwidth	;
-media	;
-meta	;
-middle	;
-multiple	;
-name	;
-nohref	;
-none	;
-noresize	;
-noshade	;
-nowrap	;
-object	;
-onblur	;
-onchange	;
-onclick	;
-ondblclick	;
-onfocus	;
-onmousedown	;
-onmouseup	;
-onmouseover	;
-onmousemove	;
-onmouseout	;
-onkeypress	;
-onkeydown	;
-onkeyup		;
-onload	;
-onselect	;
-onunload	;
-param	;
-poly	;
-profile	;
-prompt	;
-readonly	;
-rect	;
-rel	;
-rev	;
-rhs	;
-right	;
-rows	;
-rowspan	;
-rules	;
-samp	;
-scheme	;
-scope	;
-script	;
-scrolling	;
-select	;
-selected	;
-shape	;
-size	;
-small	;
-span	;
-src	;
-standby	;
-strike	;
-strong	;
-style	;
-sub	;
-summary	;
-sup	;
-tabindex	;
-table	;
-target	;
-textarea	;
-title	;
-top	;
-type	;
-usemap	;
-valign	;
-value	;
-valuetype	;
-var	;
-vlink	;
-vsides	;
-void	;
-vspace	;
-width	;
-\<\!--	;
+^From\ 						{BEGIN(HEADER); return(FROM);}
+^[A-Za-z0-9-]*Date:.*				|
+^Content-[Ii][Dd]:.*				|
+^Content-MD5:.*					|
+^In-[Rr]eply-[Tt]o:.*				|
+^Message-[Ii][Dd]:.*				|
+^References:.*					|
+^Thread-[Ii]ndex:.*				|
+^X-UIDL:.*					{if(YY_START == HEADER) {BEGIN(UNWANTED);} /* eat token anyway */}
+<UNWANTED>^[[:blank:]].*			;
+<HEADER,UNWANTED>^[A-Za-z0-9-]+:		{BEGIN(HEADER); return(TOKEN);}
+^\n						{BEGIN(INITIAL); /* eat token */}
+
+ABBR	|
+ABOVE	|
+ACRONYM	|
+ACCESSKEY	|
+ALIGN	|
+ALL	|
+ALT	|
+ALINK	|
+APPLET	|
+ARCHIVE	|
+AXIS	|
+BASEFONT	|
+BASELINE	|
+BELOW	|
+BGCOLOR	|
+BIG	|
+BODY	|
+BORDER	|
+BOTTOM	|
+BOX	|
+BUTTON	|
+CELLPADDING	|
+CELLSPACING	|
+CENTER	|
+CHAR	|
+CHAROFF	|
+CHARSET	|
+CIRCLE	|
+CITE	|
+COLSPAN	|
+COORDS	|
+CLASS	|
+CLASSID	|
+CLEAR	|
+CODEBASE	|
+CODETYPE	|
+COLOR	|
+COLS	|
+COMPACT	|
+CONTENT	|
+DATETIME	|
+DECLARE	|
+DEFER	|
+DATA	|
+DEFAULT	|
+DFN	|
+DIR	|
+DISABLED	|
+FACE	|
+FONT	|
+FRAMEBORDER	|
+GROUPS	|
+HEAD	|
+HEADERS	|
+HEIGHT	|
+HREF	|
+HREFLANG	|
+HSIDES	|
+HTTP-EQUIV	|
+HSPACE	|
+IFRAME	|
+INPUT	|
+IMG	|
+ISMAP	|
+JUSTIFY	|
+KBD	|
+LABEL	|
+LANG	|
+LANGUAGE	|
+LEFT	|
+LHS	|
+LINK	|
+LONGDESC	|
+MAP	|
+MARGINHEIGHT	|
+MARGINWIDTH	|
+MEDIA	|
+META	|
+MIDDLE	|
+MULTIPLE	|
+NAME	|
+NOHREF	|
+NONE	|
+NORESIZE	|
+NOSHADE	|
+NOWRAP	|
+OBJECT	|
+ONBLUR	|
+ONCHANGE	|
+ONCLICK	|
+ONDBLCLICK	|
+ONFOCUS	|
+ONMOUSEDOWN	|
+ONMOUSEUP	|
+ONMOUSEOVER	|
+ONMOUSEMOVE	|
+ONMOUSEOUT	|
+ONKEYPRESS	|
+ONKEYDOWN	|
+ONKEYUP	|
+ONLOAD	|
+ONSELECT	|
+ONUNLOAD	|
+PARAM	|
+POLY	|
+PROFILE	|
+PROMPT	|
+READONLY	|
+RECT	|
+REL	|
+REV	|
+RHS	|
+RIGHT	|
+ROWS	|
+ROWSPAN	|
+RULES	|
+SAMP	|
+SCHEME	|
+SCOPE	|
+SCRIPT	|
+SCROLLING	|
+SELECT	|
+SELECTED	|
+SHAPE	|
+SIZE	|
+SMALL	|
+SPAN	|
+SRC	|
+STANDBY	|
+STRIKE	|
+STRONG	|
+STYLE	|
+SUB	|
+SUMMARY	|
+SUP	|
+TABINDEX	|
+TABLE	|
+TARGET	|
+TEXTAREA	|
+TITLE	|
+TOP	|
+TYPE	|
+USEMAP	|
+VALIGN	|
+VALUE	|
+VALUETYPE	|
+VAR	|
+VLINK	|
+VSIDES	|
+VOID	|
+VSPACE	|
+WIDTH	|
+abbr	|
+above	|
+acronym	|
+accesskey	|
+align	|
+all	|
+alt	|
+alink	|
+applet	|
+archive	|
+axis	|
+basefont	|
+baseline	|
+below	|
+bgcolor	|
+big	|
+body	|
+border	|
+bottom	|
+box	|
+button	|
+cellpadding	|
+cellspacing	|
+center	|
+char	|
+charoff	|
+charset	|
+circle	|
+cite	|
+colspan	|
+coords	|
+class	|
+classid	|
+clear	|
+codebase	|
+codetype	|
+color	|
+cols	|
+compact	|
+content	|
+datetime	|
+declare	|
+defer	|
+data	|
+default	|
+dfn	|
+dir	|
+disabled	|
+face	|
+font	|
+frameborder	|
+groups	|
+head	|
+headers	|
+height	|
+href	|
+hreflang	|
+hsides	|
+http-equiv	|
+hspace	|
+iframe	|
+input	|
+img	|
+ismap	|
+justify	|
+kbd	|
+label	|
+lang	|
+language	|
+left	|
+lhs	|
+link	|
+longdesc	|
+map	|
+marginheight	|
+marginwidth	|
+media	|
+meta	|
+middle	|
+multiple	|
+name	|
+nohref	|
+none	|
+noresize	|
+noshade	|
+nowrap	|
+object	|
+onblur	|
+onchange	|
+onclick	|
+ondblclick	|
+onfocus	|
+onmousedown	|
+onmouseup	|
+onmouseover	|
+onmousemove	|
+onmouseout	|
+onkeypress	|
+onkeydown	|
+onkeyup	|
+onload	|
+onselect	|
+onunload	|
+param	|
+poly	|
+profile	|
+prompt	|
+readonly	|
+rect	|
+rel	|
+rev	|
+rhs	|
+right	|
+rows	|
+rowspan	|
+rules	|
+samp	|
+scheme	|
+scope	|
+script	|
+scrolling	|
+select	|
+selected	|
+shape	|
+size	|
+small	|
+span	|
+src	|
+standby	|
+strike	|
+strong	|
+style	|
+sub	|
+summary	|
+sup	|
+tabindex	|
+table	|
+target	|
+textarea	|
+title	|
+top	|
+type	|
+usemap	|
+valign	|
+value	|
+valuetype	|
+var	|
+vlink	|
+vsides	|
+void	|
+vspace	|
+width	|
+\<\!--	|
 -->	;
 
-^From\ 						{past_header = 0; return(FROM);}
-^Date:.*|Delivery-Date:.*			;
-^Message-ID:.*					;
 {BASE64}					;
+{PGPSIG}					;
 ^\tid\ .*					;
-
 E?SMTP\ id\ .*					;
+[\ \t]id\ <?[A-Za-z0-9]+>?			;
 boundary=.*					;
 name=\"?					;
 filename=\"?					;
-{MIME_BOUNDARY}(--)?$				{return (BOUNDARY);}
+{MIME_BOUNDARY}[[:blank:]]*$			{return (BOUNDARY);}
 
 {IPADDR}					{return(TOKEN);}
 [^[:blank:][:cntrl:][:digit:][:punct:]][^][:blank:]<>;=():&%$#@!+|/\\{}^\"?\*,[:cntrl:][]+[^[:blank:][:punct:][:cntrl:]]	{return(TOKEN);}
 .						;
-^\n						{past_header = 1; /* eat token */}
 \n						;
 
 %%
@@ -282,7 +453,7 @@
     returned = lexer_fgets(buf, max_size, yyin);
 
     while (returned != -1
-	    && !past_header
+	    && YY_START != INITIAL
 	    && memcmp(buf,spam_header_name,hdrlen) == 0)
     {
 	do {
@@ -361,5 +532,6 @@
  * The following sets edit modes for GNU EMACS
  * Local Variables:
  * mode:c
+ * indent-tabs-mode:t
  * End:
  */
